/*
 * (C) Copyright 2010
 * Texas Instruments, <www.ti.com>
 *
 * Author :
 *	Aneesh V	<aneesh@ti.com>
 *	Steve Sakoman	<steve@sakoman.com>
 *  Asad Akbar	  <asadakbar@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/arch/sys_proto.h>

/**
 * Device types
 */
#define DEVICE_TYPE_GP				  	0xF0
#define DEVICE_TYPE_HS				  	0xF1	/* TODO: It has to be updated. */
#define DEVICE_TYPE_EMU				 	0xF2	/* TODO: It has to be updated. */
#define DEVICE_TYPE_TST				 	0xF3	/* TODO: It has to be updated. */

/* Map registers. */
struct omap443x *omap443x_info = (struct omap443x *)OMAP443x_DEV_IDENTIFICATION;

/*
 *  get_device_type(): tell if GP/HS/EMU/TST
 */
u32 get_device_type(void)
{
	u32 val32;

	/* Read PROD_ID_0 register. */
	val32 = readl(&omap443x_info->prod_id_0);

	/* Get only device type bits. */
	val32 &= 0xFF;

	return (val32);
}

/*
 * get_board_rev() - get board revision
 */
u32 get_board_rev(void)
{
	return 0x20;
}

/*
 * Print CPU information
 */
int print_cpuinfo(void)
{
	u32 val32;
	u32 id[4] = { 0 };

	/* Read ID_CODE register. */
	val32 = readl(&omap443x_info->id_code);

	/* Make sure that it is as per documentation. */
	if (val32 & 0x1) {
		/**
		 * Get silicon type.
		 */
		if (val32 == 0x0B85202F) {
			printf("Silicon Type:               OMAP4430 ES1.0\n");
		} else if (val32 == 0x1B85202F) {
			printf("Silicon Type:               OMAP4430 ES2.0\n");
		} else if (val32 == 0x3B95C02F)	{
			printf("Silicon Type:               OMAP4430 ES2.1\n");
		} else {
			printf("Silicon Type:               Unknown\n");
		}

		/**
		 * Get version.
		 */
		u8 version = (val32 >> 28) & 0xFF;

		switch(version) {
		case 0:
			printf("Version:                    OMAP4430 ES1.0\n");
			break;
		case 1:
			printf("Version:                    OMAP4430 ES2.0\n");
			break;
		case 3:
			printf("Version:                    OMAP4430 ES2.1\n");
			break;
		default:
			printf("Version:                    Unknown\n");
			break;
		}

		/**
		 * Get Hawkeye
		 */
		u16 hawkeye = (val32 >> 12) & 0xFFFFF;

		if (hawkeye == 0xB852) {
			printf("Hawkeye:                    OMAP4430 ES1.0 and ES2.0\n");
		} else if (hawkeye == 0xB95C) {
			printf("Hawkeye:                    OMAP4430 ES2.1\n");
		}

		/**
		 * Get TI IDM.
		 */
		printf("Manufacturer Identity (TI): 0x%X\n", (val32 >> 1));
	} else {
		printf("%s\n", "omap_silicon_revision: TI_IDM bit is not found.");
	}

	/* Get device type. */
	u8 device_type = get_device_type();

	/* Print device type. */
	switch (device_type) {
	case DEVICE_TYPE_TST:
		printf("Device Type:                %s\n", "TST");
		break;
	case DEVICE_TYPE_EMU:
		printf("Device Type:                %s\n", "EMU");
		break;
	case DEVICE_TYPE_HS:
		printf("Device Type:                %s\n", "HS");
		break;
	case DEVICE_TYPE_GP:
		printf("Device Type:                %s\n", "GP");
		break;
	default:
		printf("Device Type:                %s\n", "???");
	}

	/* Get Die-id */
	id[0] = readl(&omap443x_info->die_id_0);
	id[1] = readl(&omap443x_info->die_id_1);
	id[2] = readl(&omap443x_info->die_id_2);
	id[3] = readl(&omap443x_info->die_id_3);

	printf("Die-ID:                     %08X-%08X-%08X-%08X\n", id[3], id[2], id[1], id[0]);

	/* Get prod-id */
	id[0] = readl(&omap443x_info->prod_id_0);
	id[1] = readl(&omap443x_info->prod_id_1);

	printf("Prod-id:                    %08X-%08X\n", id[1], id[0]);

	return 0;
}
