/*
 * (C) Copyright 2011
 * Asad Akbar     <asadakbar@gmail.com
 *
 * This file is released under the terms of GPL v2 and any later version.
 * See the file COPYING in the root directory of the source tree for details.
 */

#include <common.h>
#include <asm/arch/cpu.h>
#include <asm/io.h>
#include "board_rev.h"

/* GPIO pins for detection of the revision. */
#define GPIO_BOARD_ID0      182
#define GPIO_BOARD_ID1      101
#define GPIO_BOARD_ID2      171

/* Base addresses of GPIO banks. */
#define OMAP4_GPIO_BANK0    0x4A310000
#define OMAP4_GPIO_BANK1    0x48055000
#define OMAP4_GPIO_BANK2    0x48057000
#define OMAP4_GPIO_BANK3    0x48059000
#define OMAP4_GPIO_BANK4    0x4805B000
#define OMAP4_GPIO_BANK5    0x4805D000

/* Register for data-in. */
#define OMAP4_GPIO_DATAIN   0x138

static u32 get_gpio_index(int gpio);
static u32 gpio_get_datain(int gpio);

/* Bit position of the gpio. */
static u32 get_gpio_index(int gpio)
{
    return gpio & 0x1f;
}

static u32 gpio_get_datain(int gpio)
{
    u32 val32 = 0;

    int bank =  gpio / 32;

    if (bank == 0)
        val32 = __raw_readl(OMAP4_GPIO_BANK0 + OMAP4_GPIO_DATAIN);
    else if (bank == 1)
        val32 = __raw_readl(OMAP4_GPIO_BANK1 + OMAP4_GPIO_DATAIN);
    else if (bank == 2)
        val32 = __raw_readl(OMAP4_GPIO_BANK2 + OMAP4_GPIO_DATAIN);
    else if (bank == 3)
        val32 = __raw_readl(OMAP4_GPIO_BANK3 + OMAP4_GPIO_DATAIN);
    else if (bank == 4)
        val32 = __raw_readl(OMAP4_GPIO_BANK4 + OMAP4_GPIO_DATAIN);
    else if (bank == 5)
        val32 = __raw_readl(OMAP4_GPIO_BANK5 + OMAP4_GPIO_DATAIN);

    val32 = (val32 & (1 << get_gpio_index(gpio))) != 0;

    return val32;
}

u32 get_pandaboard_rev(void)
{
    int board_revision;

    board_revision = gpio_get_datain(GPIO_BOARD_ID0);
    board_revision |= (gpio_get_datain(GPIO_BOARD_ID1) << 1);
    board_revision |= (gpio_get_datain(GPIO_BOARD_ID2) << 2);

    return board_revision;
}
